local SynergyRT = LibStub("AceAddon-3.0"):NewAddon("SynergyRT", "AceConsole-3.0", "AceEvent-3.0")

local AceGUI = LibStub("AceGUI-3.0")
local AceTimer = LibStub("AceTimer-3.0")
local BB = LibStub("LibBabble-Boss-3.0"):GetUnstrictLookupTable()
local LDBIcon = LibStub("LibDBIcon-1.0", true)
local LDB = LibStub("LibDataBroker-1.1", true)

local selectedevent = nil -- Selected Event in Menu
local divider = ("#"):rep(10) -- Divider for Announcement to Guildchat
local lastKill = nil -- Name of the last killed Boss

local itemID = nil
local LinkBox = nil

Orig_ChatEdit_InsertLink = ChatEdit_InsertLink
function ChatEdit_InsertLink(text)
  if text and LinkBox and LinkBox:IsVisible() then
    LinkBox:SetText(text)
	itemID = text
    return true
  else
    Orig_ChatEdit_InsertLink(text)
  end
end

-- Those are ignored (Marks, Emblems, Gems, etc.)
local ignoredItems = {
	[40752] = true, -- Emblem of Heroism
	[40753] = true, -- Emblem of Valor
	[45624] = true, -- Emblem of Conquest
	[34057] = true -- Abyss Crystal
	
}

-- Ignored as well (Sartharion Adds)
local ignoredBosses = {
	[BB["Tenebron"]]= true,
	[BB["Shadron"]]= true,
	[BB["Vesperon"]]= true,
	[BB["Stalagg"]]= true,
	[BB["Feugen"]]= true,
	[BB["Death Knight Understudy"]]= true
}

SynergyRT.options = {
	desc = "Synergy Raid Tracker",
	type = "group",
	args = {
		show = {
			name = "show",
			desc = "Show the SynergyRT window",
			type = "execute",
			func = function()
				if SynergyRT.frame and SynergyRT.frame:IsShown() then
					SynergyRT.frame:Hide();
				end
				SynergyRT:ShowGUI()
			end,
		},
		announce = {
			name = "announce",
			desc = "Toggle Announcement to Guild on/off",
			type = "toggle",
			get = function(info) return SynergyRT.db.profile.announce end,
			set = function(info,val) SynergyRT.db.profile.announce = val SynergyRT:Print("Announcement set to: "..tostring(val)) end,
		},
		attendance = {
			name = "attendance",
			desc = "Creates an extra event for Attendance-DKP",
			type = "execute",
			func = function()
				SynergyRT:AddAttendanceEntry()
			end,
		},
		clear = {
			name = "clear",
			desc = "Clear all data",
			type = "execute",
			func = function()
				SynergyRT:ClearAll()
				if SynergyRT.frame and SynergyRT.frame:IsShown() then
					SynergyRT.frame:Hide();
					SynergyRT:ShowGUI()
				end
			end,
		},
		difficulty = {
			name = "difficulty",
			desc = "Needed dungeon difficulty. (Heroic = true, doesn't matter = false",
			type = "toggle",
			get = function(info) return SynergyRT.db.profile.difficulty end,
			set = function(info,val) SynergyRT.db.profile.difficulty = val SynergyRT:Print("Difficulty set to: "..tostring(val)) end,
		},
		members = {
			name = "members",
			desc = "Minimum Number of Raidmembers",
			type = "input",
			set = function(info,val)
						SynergyRT.db.profile.members = tonumber(val)
						SynergyRT:Print("Members set to: "..tostring(val))
						SynergyRT:RAID_ROSTER_UPDATE()
			end
		},
		minimap = {
			name = "minimap",
			desc = "Toggle Minimap Icon",
			type = "toggle",
			get = function(info) return SynergyRT.db.profile.minimap end,
			set = function(info,val)
						SynergyRT.db.profile.minimap = val
						if val then
							LDBIcon:Show("SynergyRT")
						else
							LDBIcon:Hide("SynergyRT")
						end
			end,
		},
		popup = {
			name = "popup",
			desc = "Toggle Loot-DKP Popup",
			type = "toggle",
			get = function(info) return SynergyRT.db.profile.popup end,
			set = function(info,val) SynergyRT.db.profile.popup = val SynergyRT:Print("Popup set to: "..tostring(val)) end,
		},
		standby = {
			type = "toggle",
			name = "Standby",
			desc = "Toggles SynergyRT on/off",
			get = function(info) return SynergyRT.db.profile.standby end,
			set = function(info, val)
				SynergyRT.db.profile.standby = val
				if val then
					SynergyRT:Disable()
				else
					SynergyRT:Enable()					
				end
			end,
		},
		wipe = {
			name = "wipe",
			desc = "Toggle WipeCheck",
			type = "toggle",
			get = function(info) return SynergyRT.db.profile.wipe end,
			set = function(info,val) SynergyRT.db.profile.wipe = val SynergyRT:Print("WipeCheck set to: "..tostring(val)) end,
		},
	},
}

local defaults = {
	profile = {
		event = {},
		announce = false,
		popup = true,
		difficulty = false,
		currentTrackSet = nil, -- Last created Trackset
		currentAnnounceSet = nil,
		raidActive = nil,
		members = 11,
		standby = false,
		minimap = false,
		minimapIcon = {},
		wipe = true
	}
}

function SynergyRT:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("SynergyRTDB", defaults, "profile")
	LibStub("AceConfig-3.0"):RegisterOptionsTable("SynergyRT", self.options)
	LibStub("AceConfigCmd-3.0"):CreateChatCommand("synergyrt", "SynergyRT")
	
	if LDB then
		self.ldb = LDB:NewDataObject("SynergyRT",
				{
					type =  "launcher", 
					label = "SynergyRT\n"..
							"|cffffff00Left click|r to open the GUI.\n"..
							"|cffffff00Right click|r to clear tracked data.",
					icon = "Interface\\Addons\\SynergyRT\\icon",
					OnClick = function(clickedframe, button)
						if button == "LeftButton" then
							if self.frame and self.frame:IsShown() then
								self.frame:Hide();
							end
							self:ShowGUI()
						elseif button == "RightButton" then
							self:ClearAll()
							if self.frame and self.frame:IsShown() then
								self.frame:Hide();
								self:ShowGUI()
							end
						end
					end,
				})
		if LDBIcon then
			LDBIcon:Register("SynergyRT", self.ldb, self.db.profile.minimapIcon)
			if not self.db.profile.minimap then
				LDBIcon:Hide("SynergyRT")
			end
		end
	end
	
	if self.db.raidActive then
		self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
		self:RegisterEvent("CHAT_MSG_LOOT")
		self:Print("is still tracking!")
	end
	
	if self.db.profile.standby then
		self:Disable()
	else
		self:Enable()
	end
end

function SynergyRT:OnEnable()
	self:RegisterEvent("RAID_ROSTER_UPDATE")
	if self.db.profile.wipe then
		self:RegisterEvent("PLAYER_REGEN_ENABLED")
	end
	self:RAID_ROSTER_UPDATE()
	self:Print("is now enabled")
end

function SynergyRT:OnDisable()
	self:UnregisterAllEvents()
	self.db.raidActive = nil
	self.db.profile.currentTrackSet = nil
	self:Print("is now disabled")
end

function SynergyRT:RAID_ROSTER_UPDATE()
	local num = GetNumRaidMembers()
	
	local id, inBG = false
	for i=1, MAX_BATTLEFIELD_QUEUES do
		_, _, id = GetBattlefieldStatus(i);
		if( id > 0 ) then
			inBG = true
		end
	end
	if not inBG and ((self.db.profile.difficulty and GetCurrentDungeonDifficulty() == 2) or not self.db.profile.difficulty) then
		-- Only check, if enough players in raid
		if not self.db.raidActive and num >= self.db.profile.members then
			self.db.raidActive = true
			GuildRoster()
			self:Print("Tracking activated.")
			self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
			self:RegisterEvent("CHAT_MSG_LOOT")
		elseif self.db.raidActive and num < self.db.profile.members then
			self.db.raidActive = nil
			self:Print("Tracking deactivated.")
			self:UnregisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
			self:UnregisterEvent("CHAT_MSG_LOOT")
			self.db.profile.currentTrackSet = nil
		end
	end
end

local function getGameTimeAsUnix()
	return time()
end

function SynergyRT:FromUnixTime(timestamp)
   local dt = date("*t", timestamp)
   return string.format("%02d/%02d/%d %02d:%02d:%02d", dt.month, dt.day, dt.year, dt.hour, dt.min, dt.sec)
end

local function whisperHandler()
	local player = arg2

	local text = arg1:lower():trim():gsub("^%l", string.upper)
	local set = SynergyRT.db.profile.event[SynergyRT.db.profile.currentAnnounceSet]
	if set then
		local found = nil
		local inGuild = nil
		local class = nil
		local level = nil
		for i, v in ipairs(set.attendants) do
			if v == text then
				found = true
				SendChatMessage(format("<SynergyRT>: %q ist bereits eingetragen.",text), "WHISPER", nil, player)
				break
			end
		end
		if not found then
			for i=1,GetNumGuildMembers(true) do
				if text == GetGuildRosterInfo(i) then
					_,_,_,level,class = GetGuildRosterInfo(i)
					inGuild = true
					break
				end
			end
		end
		if not inGuild then
			SendChatMessage(format("<SynergyRT>: %q ist kein Mitglied von 'Synergy'.",text), "WHISPER", nil, player)
		end
		if inGuild and not found then
			table.insert(set.attendants, {['name']=text, ['class']=class, ['level']=level})
			SendChatMessage(format("<SynergyRT>: %q wurde dem Bosskill hinzugef\195\188gt.",text), "WHISPER", nil, player)
			SynergyRT:Print(format("%q hinzugef\195\188gt von %s.",text, player))
			return true
		end
	end
end

local function whisperInformHandler(msg)
	if msg:find("^<SynergyRT>") then return true end
end

-- Get Raidmembers
function SynergyRT:TakeAttendance(wipe)
	local newTrackset = {
		name = lastKill,
		id = GetInstanceID(),
		zone = GetRealZoneText(),
		attendants = {},
		date = getGameTimeAsUnix(),
		loot = {},
	}
	if wipe then
		newTrackset.name = UnitName("target").." Wipe"
	end

	for i = 1, GetNumRaidMembers() do
		local name,_,group = GetRaidRosterInfo(i)
		local class = UnitClass("raid"..i)
		local race = UnitRace("raid"..i)
		local level = UnitLevel("raid"..i)
		if group < 6 or not self.db.profile.announce then
			table.insert(newTrackset.attendants, {['name']=name, ['class']=class, ['race']=race, ['level']=level})
		end
	end
	table.insert(self.db.profile.event, newTrackset)
	self.db.profile.currentTrackSet = #self.db.profile.event
	self.db.profile.currentAnnounceSet = #self.db.profile.event

	if self.db.profile.announce then
		ChatFrame_AddMessageEventFilter("CHAT_MSG_WHISPER", whisperHandler)
		ChatFrame_AddMessageEventFilter("CHAT_MSG_WHISPER_INFORM", whisperInformHandler)

		local str = nil
		str = format("SynergyRT: %s down! Ersatzb\195\164nkler mir den Namen ihres Mainchars whispern. Ihr habt 3 Minuten Zeit!",lastKill)
		SendChatMessage(divider, "GUILD")
		SendChatMessage(str, "GUILD")
		SendChatMessage(divider, "GUILD")

		AceTimer:ScheduleTimer(self.AttendanceTimeout, 180)
	end
end

function SynergyRT:AttendanceTimeout()
	ChatFrame_RemoveMessageEventFilter("CHAT_MSG_WHISPER", whisperHandler)
	ChatFrame_RemoveMessageEventFilter("CHAT_MSG_WHISPER_INFORM", whisperInformHandler)

	local str = "SynergyRT: Ersatzbankerfassung vorbei!"
	SendChatMessage(str, "GUILD")
end

function SynergyRT:PLAYER_REGEN_ENABLED()
	local BigWigs = _G["BigWigs"]
	if not UnitIsFeignDeath("player") then
		local go = BigWigs.modulePrototype:Scan()
		if not go then
			self:IsBossWipe()
			return
		end
	end

	if not UnitAffectingCombat("player") then
		self:ScheduleEvent(self.PLAYER_REGEN_ENABLED, 2, self)
	end
end

local linkpat = '|c........|Hitem:(%%d+):.-|h|r'
local globalpatterns = {
   "LOOT_ITEM",
   "LOOT_ITEM_SELF",
}
local patterns = {}
for _, global in ipairs(globalpatterns) do
   local pattern = _G[global]
   if strsub(pattern, 1,2) == "%s" then
      pattern = "([^ ]+)"..strsub(pattern, 3)
   end
   patterns[global] = gsub(gsub(pattern, "%%d", "(%%d+)"), "%%s", linkpat)
end

-- Check for Loot
function SynergyRT:CHAT_MSG_LOOT(_,msg)
   player, item = strmatch(msg, patterns["LOOT_ITEM"])
   if player and item then
      self:ItemLooted(player, item)
      return
   end

   item = strmatch(msg, patterns["LOOT_ITEM_SELF"])
   if item then
      self:ItemLooted(UnitName("player"), item)
      return
   end
end

-- Check for Bosskill
function SynergyRT:COMBAT_LOG_EVENT_UNFILTERED(_, _, event, _, _, _, _, mob)
	if event ~= "UNIT_DIED" and event ~= "UNIT_DESTROYED" then return end
	if mob and not ignoredBosses[mob] and BB[mob] then
		self:Print(format("%s down. Erfasse Anwesenheit...",mob))
		lastKill = mob
		self:TakeAttendance()
	end
end

-- Clear all Data
function SynergyRT:ClearAll()
	self.db.profile.currentTrackSet = nil
	self.db.profile.generic = nil
	self.db.profile.event = {}
	lastKill = nil
	
	self:Print("All data cleared!")
end

-- Get Item and save into DB
function SynergyRT:ItemLooted(player, item)
	local _, itemLink, itemRarity = GetItemInfo(item)
	
	local itemID = select(3, itemLink:find("item:(%d+):"))
	if not itemID then return false end
	itemID = tonumber(itemID:trim())
	if type(itemID) ~= "number" then return false end
	if ignoredItems[itemID] or (itemRarity < 4 and not ignoredItems[itemID]) then return false end
	
	table.insert(self.db.profile.event[self.db.profile.currentTrackSet].loot, {
		player = player,
		item = itemID,
		dkp = 0,
	})
	
	if self.db.profile.popup then
		self:GetDKP()
	end
end

-- Convert selected kill to EQDKP-String
function SynergyRT:XMLOutput(event)
	local text = "<RaidInfo>"
	local players_join = ""
	local players_leave = ""

	text = text .. "<key>" .. self:FromUnixTime(event.date) .. "</key>"
	text = text .. "<start>" .. self:FromUnixTime(event.date) .. "</start>"
	text = text .. "<end>" .. self:FromUnixTime(event.date+1) .. "</end>"
	text = text .. "<zone>" .. event.zone .. "</zone>"
	text = text .. "<note>" .. event.zone .. "(" ..  event.name .. ")</note>"

	text = text .. "<PlayerInfos>"

	for i, v in ipairs(event.attendants) do
		text = text .. "<key" .. i .. "><name>" .. v.name .. "</name>"
		if v.class then text = text .. "<class>" .. v.class .. "</class>" end
		if v.level then text = text .. "<level>" .. v.level .. "</level>" end
		if v.race then text = text .. "<race>" .. v.race .. "</race>" end
		text = text .. "</key" .. i .. ">"
		players_join = players_join .. "<key" .. i .. "><player>" .. v.name .. "</player><time>" .. self:FromUnixTime(event.date) .. "</time></key" .. i .. ">"
		players_leave = players_leave .. "<key" .. i .. "><player>" .. v.name .. "</player><time>" .. self:FromUnixTime(event.date+1) .. "</time></key" .. i .. ">"
	end

	text = text .. "</PlayerInfos>"

	text = text .. "<Bossevent>"
	
	text = text .. "<key" .. 1 .. "><name>" .. event.name .. "</name><time>" .. self:FromUnixTime(event.date) .. "</time><attendees/></key" .. 1 .. ">"

	text = text .. "</Bossevent>"

	text = text .. "<Join>" .. players_join .. "</Join>"
	text = text .. "<Leave>" .. players_leave .. "</Leave>"

	text = text .. "<Loot>"


	for i, item in ipairs(event.loot) do
		local itemName,_,rarity = GetItemInfo(item.item)
		local id = item.item
		local _,_,_,hex = GetItemQualityColor(rarity)
		text = text .. "<key" .. i .. ">"
		text = text .. "<ItemName>" .. (itemName or "?") .. "</ItemName>"
		text = text .. "<ItemID>" .. id .. "</ItemID>"
		text = text .. "<Color>" .. strsub(hex,3) .. "</Color>"
		text = text .. "<Count>1</Count>"
		text = text .. "<Player>" .. item.player .. "</Player>"
		text = text .. "<Note>" .. item.dkp .. " DKP</Note>"
		text = text .. "<Time>" .. self:FromUnixTime(event.date) .. "</Time>"
		text = text .. "<Zone>" .. event.zone .. "</Zone>"
		text = text .. "<Boss>" .. event.name .. "</Boss>"
		text = text .. "</key" .. i .. ">"
	end

	text = text .. "</Loot>"
	text = text .. "</RaidInfo>"
	return text
end

function SynergyRT:SelectItem(widget, event, value)
	widget:ReleaseChildren()
	
	local sf = AceGUI:Create("ScrollFrame")
	sf:SetLayout("List")
	
	local eItemID = AceGUI:Create("EditBox")
	eItemID:SetLabel("ItemID")
	eItemID:SetText(self.db.profile.event[selectedevent].loot[value].item)
	eItemID:SetCallback("OnEnterPressed",function(widget,event,text) self.db.profile.event[selectedevent].loot[value].item = strtrim(text) end )
	
	local eLooter = AceGUI:Create("EditBox")
	eLooter:SetLabel("Looted by")
	eLooter:SetText(self.db.profile.event[selectedevent].loot[value].player)
	eLooter:SetCallback("OnEnterPressed",function(widget,event,text) self.db.profile.event[selectedevent].loot[value].player = strtrim(text) end )
	
	local bDisenchanted = AceGUI:Create("Button")
	bDisenchanted:SetText("D")
	bDisenchanted:SetWidth(40)
	bDisenchanted:SetCallback("OnClick", function()
								self.db.profile.event[selectedevent].loot[value].player = "disenchanted"
								eLooter:SetText("disenchanted")
							end )
	local bBank = AceGUI:Create("Button")
	bBank:SetText("B")
	bBank:SetWidth(40)
	bBank:SetCallback("OnClick", function()
								self.db.profile.event[selectedevent].loot[value].player = "bank"
								eLooter:SetText("bank")
							end )
	local bTarget = AceGUI:Create("Button")
	bTarget:SetText("T")
	bTarget:SetWidth(40)
	bTarget:SetCallback("OnClick", function()
								self.db.profile.event[selectedevent].loot[value].player = UnitName("target")
								eLooter:SetText(UnitName("target"))
							end )
	
	local eDKP = AceGUI:Create("EditBox")
	eDKP:SetLabel("DKP")
	eDKP:SetText(self.db.profile.event[selectedevent].loot[value].dkp)
	eDKP:SetCallback("OnEnterPressed",function(widget,event,text) self.db.profile.event[selectedevent].loot[value].dkp = strtrim(text) end )
	
	local bRemove = AceGUI:Create("Button")
	bRemove:SetText("Item entfernen")
	bRemove:SetCallback("OnClick", function()
								table.remove(self.db.profile.event[selectedevent].loot, value)
								eItemID:SetText("")
								eLooter:SetText("")
								eDKP:SetText("")
								
								local tree = {}
								for i,v in ipairs(self.db.profile.event[selectedevent].loot) do 
									table.insert(tree, {value = i, text = GetItemInfo(v.item)})
								end
								
								widget:SetTree(tree)
							end )
	sf:AddChild(eItemID)
	sf:AddChild(eLooter)
	sf:AddChild(bDisenchanted)
	sf:AddChild(bBank)
	sf:AddChild(bTarget)
	sf:AddChild(eDKP)
	sf:AddChild(bRemove)
	widget:AddChild(sf)
end
	
	
function SynergyRT:SelectEvent(widget, event, value)
	widget:ReleaseChildren()
	selectedevent = value
	
	local sf = AceGUI:Create("ScrollFrame")
	sf:SetLayout("Flow")
	
	local hEvent = AceGUI:Create("Heading")
	hEvent:SetText("Event")
	hEvent.width = "fill"
	
	local sg0 = AceGUI:Create("SimpleGroup")
	sg0:SetWidth(200)
	local sg1 = AceGUI:Create("SimpleGroup")
	sg1:SetLayout("Flow")
	local eZone = AceGUI:Create("EditBox")
	eZone:SetLabel("Zone")
	eZone:SetWidth(150)
	eZone:SetText(self.db.profile.event[value].zone)
	eZone:SetCallback("OnEnterPressed",function(widget,event,text) self.db.profile.event[value].zone = strtrim(text) end )
	local bZone = AceGUI:Create("Button")
	bZone:SetText("T")
	bZone:SetWidth(40)
	bZone:SetCallback("OnClick", function()
								self.db.profile.event[value].zone = GetRealZoneText()
								eZone:SetText(GetRealZoneText())
							end )
	
	local sg2 = AceGUI:Create("SimpleGroup")
	sg2:SetWidth(200)
	local meAttendants = AceGUI:Create("MultiLineEditBox")
	meAttendants:SetLabel("Attendants")
	local attendees = ""
	for i,v in ipairs(self.db.profile.event[value].attendants) do
		attendees = attendees .. v.name .. "\n"
	end
	meAttendants:SetText(attendees)
	meAttendants:SetHeight(200)
	meAttendants:SetWidth(200)
	meAttendants:SetCallback("OnEnterPressed",function(widget,event,text)
										--TODO
									end )
	local bAddTarget = AceGUI:Create("Button")
	bAddTarget:SetText("T")
	bAddTarget:SetWidth(40)
	bAddTarget:SetCallback("OnClick", function()
									local name, _ = UnitName("target")
									table.insert(self.db.profile.event[value].attendants, name)
									for i,v in ipairs(self.db.profile.event[value].attendants) do
										attendees = attendees .. v .. "\n"
									end
									meAttendants:SetText(attendees)
							end )
							
	local sg3 = AceGUI:Create("SimpleGroup")
	sg3:SetLayout("Flow")
	local eBoss = AceGUI:Create("EditBox")
	eBoss:SetLabel("Boss")
	eBoss:SetWidth(150)
	eBoss:SetText(self.db.profile.event[value].name)
	eBoss:SetCallback("OnEnterPressed",function(widget,event,text) self.db.profile.event[value].name = strtrim(text) end )
	local bBossSetTarget = AceGUI:Create("Button")
	bBossSetTarget:SetText("T")
	bBossSetTarget:SetWidth(40)
	bBossSetTarget:SetCallback("OnClick", function()
								self.db.profile.event[value].zone = (UnitName("target") or GetRealZoneText())
								eBoss:SetText((UnitName("target") or GetRealZoneText()))
							end )
							
	local eDate = AceGUI:Create("EditBox")
	eDate:SetLabel("Date")
	eDate:SetWidth(150)
	eDate:SetText(self.db.profile.event[value].date)
	edate:SetCallback("OnEnterPressed",function(widget,event,text) self.db.profile.event[value].date = strtrim(text) end )
		
	local hLoot = AceGUI:Create("Heading")
	hLoot:SetText("Loot")
	hLoot.width = "fill"
	
	local tLoot = AceGUI:Create("TreeGroup")
	tLoot.width = "fill"
	tLoot:SetHeight(300)
	tLoot:SetLayout("Fill")
	
	local tree = {}
	for i,v in ipairs(self.db.profile.event[value].loot) do 
		table.insert(tree, {value = i, text = GetItemInfo(v.item)})
	end
	tLoot:SetTree(tree)
	tLoot:SetCallback("OnGroupSelected", function(w,e,v) self:SelectItem(w,e,v) end )

	local bItemAdd = AceGUI:Create("Button")
	bItemAdd:SetWidth(150)
	bItemAdd:SetText("Item hinzuf\195\188gen")
	bItemAdd:SetCallback("OnClick", function()
								_,itemID = GetItemInfo(itemID)
								if(itemID) then
									_, _, itemID = string.find(itemID, "^|c%x+|%a+:(%d+):.+|h%[.*%]")
									table.insert(self.db.profile.event[selectedevent].loot, {['item']=itemID,['player']="player",['dkp']=0})
									
									local tree = {}
									for i,v in ipairs(self.db.profile.event[selectedevent].loot) do 
										table.insert(tree, {value = i, text = GetItemInfo(v.item)})
									end
									LinkBox:SetText("")
									
									tLoot:SetTree(tree)
								else
									self:Print("Invalid Item: "..itemID)
								end
							end )

	local eItemNew = AceGUI:Create("EditBox")
	eItemNew:SetLabel("Item")
	eItemNew:SetWidth(150)
	eItemNew:SetText("")
	eItemNew:SetCallback("OnEnterPressed",function(widget,event,text) itemID = text end )
	LinkBox = eItemNew
	
	local hMisc = AceGUI:Create("Heading")
	hMisc:SetText("Misc")
	hMisc.width = "fill"
	
	local meExportString = AceGUI:Create("MultiLineEditBox")
	meExportString:SetLabel("EQDKP-String")
	meExportString:SetText("")
	meExportString.width = "fill"
	meExportString:SetHeight(200)
	
	local bExportString = AceGUI:Create("Button")
	bExportString:SetText("XML Output")
	bExportString:SetWidth(200)
	bExportString:SetCallback("OnClick", function() meExportString:SetText(self:XMLOutput(self.db.profile.event[value])) end )
	
	local bRemove = AceGUI:Create("Button")
	bRemove:SetText("Kill entfernen")
	bRemove:SetWidth(200)
	bRemove:SetCallback("OnClick", function()
								table.remove(self.db.profile.event, value)
								self.db.profile.currentTrackSet = #self.db.profile.event;
								widget.parent:Hide()
								self:ShowGUI()
							end )
	
	sf:AddChild(hEvent)
	sg1:AddChild(eZone)
	sg1:AddChild(bZone)
	sg3:AddChild(eBoss)
	sg3:AddChild(bBoss)
	sg2:AddChild(meAttendants)
	sg2:AddChild(bAddTarget)
	sg0:AddChild(sg1)
	sg0:AddChild(sg3)
	sg0:AddChild(eDate)
	sf:AddChild(sg0)
	sf:AddChild(sg2)
	sf:AddChild(hLoot)
	sf:AddChild(tLoot)
	sf:AddChild(bItemAdd)
	sf:AddChild(eItemNew)
	sf:AddChild(hMisc)
	sf:AddChild(bExportString)
	sf:AddChild(bRemove)
	sf:AddChild(meExportString)
	widget:AddChild(sf)
end

-- Create Menu
function SynergyRT:ShowGUI()
	-- The Big Black One
	local fMain = AceGUI:Create("Frame")
	fMain:SetTitle("SynergyRT Menu")
	fMain:SetLayout("Fill")
	
	self.frame = fMain
	
	local tEvents = AceGUI:Create("TreeGroup")
	tEvents:SetLayout("Fill")
	local tree = {}
	for i,v in ipairs(self.db.profile.event) do 
		table.insert(tree, {value = i, text = v.name})
	end
	fMain:SetStatusText(#tree .." entries found.")
	tEvents:SetTree(tree)
	tEvents:SetCallback("OnGroupSelected", function(w,e,v) self:SelectEvent(w,e,v) end )
	fMain:AddChild(tEvents)
end

-- Adds generic Event for current Zone
function SynergyRT:AddAttendanceEntry()
	local newTrackset = {
		name = "Anwesenheit",
		zone = GetRealZoneText(),
		attendants = {},
		date = getGameTimeAsUnix(),
		loot = {},
	}
	for i = 1, GetNumRaidMembers() do
		local name = GetRaidRosterInfo(i)
		local class = UnitClass("raid"..i)
		local race = UnitRace("raid"..i)
		local level = UnitLevel("raid"..i)
		table.insert(newTrackset.attendants, {['name']=name, ['class']=class, ['race']=race, ['level']=level})
	end
	table.insert(self.db.profile.event, newTrackset)
	self.db.profile.currentTrackSet = #self.db.profile.event
end

function SynergyRT:GetDKP()
	local lootid = #self.db.profile.event[self.db.profile.currentTrackSet].loot
	local item = GetItemInfo(self.db.profile.event[self.db.profile.currentTrackSet].loot[lootid].item)
	local player = self.db.profile.event[self.db.profile.currentTrackSet].loot[lootid].player
	
	local fDKP = AceGUI:Create("Frame")
	fDKP:SetTitle("DKP eingeben")
	fDKP:SetLayout("List")
	fDKP:SetWidth(400)
	fDKP:SetHeight(200)
	
	local sg = AceGUI:Create("SimpleGroup")
	
	local lLooter = AceGUI:Create("Label")
	lLooter:SetWidth(300)
	lLooter:SetText(player..": "..item)
	
	local eDKP = AceGUI:Create("EditBox")
	eDKP:SetLabel("DKP:")
	eDKP:SetWidth(300)
	eDKP:SetText("0")
	eDKP:SetCallback("OnEnterPressed",function(widget,event,text)
										self.db.profile.event[self.db.profile.currentTrackSet].loot[lootid].dkp = strtrim(text)
									end )
	
	local bDisenchanted = AceGUI:Create("Button")
	bDisenchanted:SetText("disenchanted")
	bDisenchanted:SetWidth(200)
	bDisenchanted:SetCallback("OnClick", function()
								self.db.profile.event[self.db.profile.currentTrackSet].loot[lootid].player = "disenchanted"
								lLooter:SetText("disenchanted: "..item)
							end )
	local bBank = AceGUI:Create("Button")
	bBank:SetText("bank")
	bBank:SetWidth(200)
	bBank:SetCallback("OnClick", function()
								self.db.profile.event[self.db.profile.currentTrackSet].loot[lootid].player = "bank"
								lLooter:SetText("bank: "..item)
							end )
	
	sg:AddChild(lLooter)
	sg:AddChild(eDKP)
	sg:AddChild(bDisenchanted)
	sg:AddChild(bBank)
	fDKP:AddChild(sg)
end

function SynergyRT:IsBossWipe()
	fWipe = AceGUI:Create("BlankFrame")
	fWipe:SetTitle("WipeCheck")
	fWipe:SetLayout("List")
	fWipe:SetWidth(200)
	fWipe:SetHeight(100)
	
	lQuestion = AceGUI:Create("Label")
	lQuestion:SetWidth(150)
	lQuestion:SetText("Wiped at a Boss?\n(Target the Boss if it is)")
	
	bYes = AceGUI:Create("Button")
	bYes:SetText("Yes")
	bYes:SetWidth(50)
	bYes:SetCallback("OnClick", function()
								self:TakeAttendance(true)
								fWipe:Hide()
							end)
	
	bNo = AceGUI:Create("Button")
	bNo:SetText("No")
	bNo:SetWidth(50)
	bNo:SetCallback("OnClick", function()
								fWipe:Hide()
							end)
end

function SynergyRT:GetInstanceID()
	for i = 1, GetNumSavedInstances() do
		local name, id, _, diff = GetSavedInstanceInfo(i)
		if name == GetRealZoneText() and diff == GetCurrentDungeonDifficulty() then
			return id
		end
	end
end
